from typing import Dict
try:
    # When running from this package
    from location_local_constants import LocationLocalConstants
except Exception:
    # When importing this module from another package
    from location_local.location_local_constants import LocationLocalConstants
from dotenv import load_dotenv
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD  # noqa: E402
from circles_local_database_python.connector import Connector   # noqa: E402
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)


class County(GenericCRUD):
    def __init__(self):
        logger.start()

        self.connector = Connector.connect("location")
        self.cursor = self.connector.cursor()

        logger.end()

    def insert(
            self, coordinate: Dict[str, float],
            county: str, lang_code: LangCode, title_approved: bool = False, state_id: int = None) -> int:
        logger.start(object={'coordinate': coordinate, 'county': county,
                     'lang_code': lang_code, 'title_approved': title_approved, 'state_id': state_id})
        insert_county_sql = "INSERT INTO county_table (coordinate) VALUES (POINT(%s, %s))"
        self.cursor.execute(
            insert_county_sql, (coordinate["latitude"], coordinate["longitude"]))
        county_id = self.cursor.lastrowid()
        insert_county_ml_sql = "INSERT INTO county_ml_table (county_id, lang_code," \
            " title, title_approved) VALUES (%s, %s, %s, %s)"
        self.cursor.execute(insert_county_ml_sql, (county_id,
                            lang_code, county, title_approved))
        self.connector.commit()
        logger.end(object={'county_id': county_id})
        return county_id

    @staticmethod
    def get_county_ids_by_county_name(county_name: str, state_id: int = None) -> list[int]:
        logger.start(object={'county_name': county_name, 'state_id': state_id})

        connector = Connector.connect("location")
        cursor = connector.cursor()

        if county_name is None:
            select_county_ml_sql = "SELECT county_id FROM county_ml_view WHERE title IS NULL"
            cursor.execute(select_county_ml_sql)
        else:
            select_county_ml_sql = "SELECT county_id FROM county_ml_view WHERE title = %s"
            cursor.execute(select_county_ml_sql, (county_name,))
        county_rows = cursor.fetchall()
        county_ids = []
        for county_row in county_rows:
            county_ids.append(county_row[0])

        logger.end(object={'county_ids': county_ids})
        return county_ids


from typing import Dict
try:
    # When running from this package
    from location_local_constants import LocationLocalConstants
except Exception:
    # When importing this module from another package
    from location_local.location_local_constants import LocationLocalConstants
from dotenv import load_dotenv
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD  # noqa: E402
from circles_local_database_python.connector import Connector   # noqa: E402
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402

logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)


class Region(GenericCRUD):
    def __init__(self):
        logger.start()

        self.connector = Connector.connect("location")
        self.cursor = self.connector.cursor()

        logger.end()

    def insert(
            self, coordinate: Dict[str, float],
            region: str, lang_code: LangCode, title_approved: bool = False, country_id: int = None) -> int:
        logger.start(object={'coordinate': coordinate, 'region': region,
                     'lang_code': lang_code, 'title_approved': title_approved, 'country_id': country_id})
        insert_region_sql = "INSERT INTO region_table (coordinate) VALUES (POINT(%s, %s))"
        self.cursor.execute(
            insert_region_sql, (coordinate["latitude"], coordinate["longitude"]))
        region_id = self.cursor.lastrowid()
        insert_region_ml_sql = "INSERT INTO region_ml_table (region_id, lang_code, title, title_approved)" \
            " VALUES (%s, %s, %s, %s)"
        self.cursor.execute(insert_region_ml_sql, (region_id,
                            lang_code, region, title_approved))
        self.connector.commit()
        logger.end(object={'region_id': region_id})
        return region_id

    @staticmethod
    def get_region_ids_by_region_name(region_name: str, country_id: int = None) -> list[int]:
        logger.start(object={'region_name': region_name, 'country_id': country_id})

        connector = Connector.connect("location")
        cursor = connector.cursor()

        if(region_name is None):
            select_region_ml_sql = "SELECT region_id FROM region_ml_view WHERE title IS NULL"
            cursor.execute(select_region_ml_sql)
        else:
            select_region_ml_sql = "SELECT region_id FROM region_ml_view WHERE title = %s"
            cursor.execute(select_region_ml_sql, (region_name,))
        region_rows = cursor.fetchall()
        region_ids = []
        for region_row in region_rows:
            region_ids.append(region_row[0])

        logger.end(object={'region_ids': region_ids})
        return region_ids



from logger_local.LoggerComponentEnum import LoggerComponentEnum


class LocationLocalConstants:
    LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_ID = 113
    LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME = 'location local python package'
    OBJECT_FOR_LOGGER_CODE = {
        'component_id': LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
        'component_name': LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
        'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
        'developer_email': 'tal.g@circ.zone'
    }

    OBJECT_FOR_LOGGER_TEST = {
        'component_id': LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
        'component_name': LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
        'component_category': LoggerComponentEnum.ComponentCategory.Unit_Test.value,
        'developer_email': 'tal.g@circ.zone'
    }

    UNKNOWN_LOCATION_ID = 37522
    DEFAULT_NEGIHBORHOOD_NAME = None
    DEFAULT_COUNTY_NAME = None
    DEFAULT_REGION_NAME = None
    DEFAULT_STATE_NAME = 'UNKNOWN'  #Cannot be None because it would result in mysql.connector.errors.IntegrityError: 1048 (23000): Column 'state_name' cannot be null
    DEFAULT_COUNTRY_NAME = 'UNKNOWN'    #Cannot be None because `name` is an index
    DEFAULT_ADDRESS_LOCAL_LANGUAGE = None
    DEFAULT_ADDRESS_ENGLISH = None
    DEFAULT_POSTAL_CODE = None
    DEFAULT_PLUS_CODE = None
    DEFAULT_COORDINATE = {'latitude': 0.0, 'longitude': 0.0}
    LOCATION_SCHEMA_NAME = 'location'
    LOCATION_TABLE_NAME = 'location_table'
    LOCATION_VIEW_NAME = 'location_view'
    COUNTRY_TABLE_NAME = 'country_table'
    COUNTRY_ML_TABLE_NAME = 'country_ml_table'
    COUNTRY_ML_VIEW_NAME = 'country_ml_view'
    COUNTY_TABLE_NAME = 'county_table'
    COUNTY_ML_TABLE_NAME = 'county_ml_table'
    COUNTY_ML_VIEW_NAME = 'county_ml_view'
    NEIGHBORHOOD_TABLE_NAME = 'neighborhood_table'
    NEIGHBORHOOD_ML_TABLE_NAME = 'neighborhood_ml_table'
    NEIGHBORHOOD_ML_VIEW_NAME = 'neighborhood_ml_view'
    REGION_TABLE_NAME = 'region_table'
    REGION_ML_TABLE_NAME = 'region_ml_table'
    REGION_ML_VIEW_NAME = 'region_ml_view'
    STATE_TABLE_NAME = 'state_table'
    STATE_ML_TABLE_NAME = 'state_ml_table'
    STATE_ML_VIEW_NAME = 'state_ml_view'





from typing import Dict
try:
    # When running from this package
    from location_local_constants import LocationLocalConstants
except Exception:
    # When importing this module from another package
    from location_local.location_local_constants import LocationLocalConstants
from dotenv import load_dotenv
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD  # noqa: E402
from circles_local_database_python.connector import Connector   # noqa: E402
from logger_local.Logger import Logger  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)


class State(GenericCRUD):
    def __init__(self):
        logger.start()

        self.connector = Connector.connect("location")
        self.cursor = self.connector.cursor()

        logger.end()

    def insert(
            self, coordinate: Dict[str, float],
            state: str, lang_code: LangCode, state_name_approved: bool = False, country_id: int = None) -> int:
        logger.start(object={'coordinate': coordinate, 'state': state,
                     'lang_code': lang_code, 'state_name_approved': state_name_approved, 'country_id': country_id})
        insert_state_sql = "INSERT INTO state_table (coordinate) VALUES (POINT(%s, %s))"
        self.cursor.execute(
            insert_state_sql, (coordinate["latitude"], coordinate["longitude"]))
        state_id = self.cursor.lastrowid()
        insert_state_ml_sql = "INSERT INTO state_ml_table (state_id, lang_code, state_name," \
            " state_name_approved) VALUES (%s, %s, %s, %s)"
        self.cursor.execute(
            insert_state_ml_sql, (state_id, lang_code, state, state_name_approved))
        self.connector.commit()
        logger.end(object={'state_id': state_id})
        return state_id

    @staticmethod
    def get_state_ids_by_state_name(state_name: str, country_id: int = None) -> list[int]:
        logger.start(object={'state_name': state_name, 'country_id': country_id})

        connector = Connector.connect("location")
        cursor = connector.cursor()

        select_state_ml_sql = "SELECT state_id FROM state_ml_view WHERE state_name = %s"
        cursor.execute(select_state_ml_sql, (state_name,))
        state_rows = cursor.fetchall()
        state_ids = []
        for state_row in state_rows:
            state_ids.append(state_row[0])

        logger.end(object={'state_ids': state_ids})
        return state_ids



from typing import Dict
from logger_local.LoggerComponentEnum import LoggerComponentEnum
try:
    # When running from this package
    from neighborhood import Neighborhood
    from county import County
    from region import Region
    from state import State
    from country import Country
except Exception:
    # When importing this module from another package
    from location_local.neighborhood import Neighborhood
    from location_local.county import County
    from location_local.region import Region
    from location_local.state import State
    from location_local.country import Country
from dotenv import load_dotenv
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD   # noqa: E402
from circles_local_database_python.connector import Connector   # noqa: E402
from logger_local.Logger import Logger  # noqa: E402
from user_context_remote.user_context import UserContext  # noqa: E402
from language_local.lang_code import LangCode  # noqa: E402

LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_ID = 113
LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME = 'location/src/location.py'


object_to_insert = {
    'component_id': LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
    'developer_email': 'tal.g@circ.zone'
}

logger = Logger.create_logger(object=object_to_insert)

user_context = UserContext().login_using_user_identification_and_password()


class LocationLocal(GenericCRUD):
    def __init__(self):
        logger.start()

        self.connector = Connector.connect("location")
        self.cursor = self.connector.cursor()

        logger.end()

    def get_location_ids(self, neighborhood_name: str, county_name: str, region_name: str, state_name: str,
                               country_name: str) -> tuple([list[int], int, int, int, int]):
        logger.start(object={'neighborhood_name': neighborhood_name, 'county_name': county_name,
                     'region_name': region_name, 'state_name': state_name, 'country_name': country_name})

        neighborhood_ids = Neighborhood.get_neighborhood_ids_by_neighborhood_name(neighborhood_name)
        county_ids = County.get_county_ids_by_county_name(county_name)
        region_ids = Region.get_region_ids_by_region_name(region_name)
        state_ids = State.get_state_ids_by_state_name(state_name)
        country_id = Country.get_country_id_by_country_name(country_name)

        logger.end(object={"neighborhood_ids": neighborhood_ids, "county_ids": county_ids, "region_ids": region_ids,
                           "state_ids": state_ids, "country_id": country_id})
        return (neighborhood_ids, county_ids, region_ids, state_ids, country_id)

    def insert(
            self, data: Dict[str, any],
            lang_code: LangCode = user_context.get_curent_lang_code(),
            is_approved: bool = True, new_country_data: Dict[str, any] = None) -> int:
        logger.start(object={'data': data, 'lang_code': lang_code,
                     'is_approved': is_approved, 'new_country_data': new_country_data})
        (neighborhood_id, county_id, region_id, state_id, country_id) = self._check_details_and_insert_if_not_exist(
            data["coordinate"],
            (data["neighborhood"],
             data["county"],
             data["region"],
             data["state"],
             data["country"]),
            lang_code, is_approved, new_country_data)

        insert_location_sql = "INSERT INTO location_table (coordinate, address_local_language, address_english," \
            " neighborhood_id, county_id, region_id, state_id, country_id, postal_code, plus_code, is_approved)" \
            " VALUES (POINT(%s, %s), %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
        self.cursor.execute(
            insert_location_sql,
            (data["coordinate"]["latitude"],
             data["coordinate"]["longitude"],
             data["address_local_language"],
             data["address_english"],
             neighborhood_id, county_id, region_id, state_id, country_id, data["postal_code"],
             data["plus_code"],
             is_approved))
        self.connector.commit()

        location_id = self.cursor.lastrowid()

        logger.end(object={'location_id': location_id})
        return location_id

    def update(self, location_id: int, data: Dict[str, any], lang_code: LangCode = "en", is_approved: bool = True):
        logger.start(object={'location_id': location_id, 'data': data,
                     'lang_code': lang_code, 'is_approved': is_approved})
        (neighborhood_id, county_id, region_id, state_id, country_id) = self._check_details_and_insert_if_not_exist(
            data["coordinate"], (data["neighborhood"], data["county"], data["region"], data["state"], data["country"]), lang_code, is_approved)

        update_location_sql = "UPDATE location_table SET coordinate = POINT(%s, %s), address_local_language = %s, " \
            "address_english = %s, neighborhood_id = %s, county_id = %s, region_id = %s, state_id = %s," \
            " country_id = %s, postal_code = %s, plus_code = %s, is_approved = %s WHERE location_id = %s"
        self.cursor.execute(
            update_location_sql,
            (data["coordinate"]["latitude"],
             data["coordinate"]["longitude"],
             data["address_local_language"],
             data["address_english"],
             neighborhood_id, county_id, region_id, state_id, country_id, data["postal_code"],
             data["plus_code"],
             is_approved,
             location_id))
        self.connector.commit()
        logger.end()

    def read(self, location_id: int) -> tuple([Dict[str, float], str, str, str, str, str, str, str, str, str]):
        logger.start(object={'location_id': location_id})

        select_location_sql = '''SELECT ST_X(l.coordinate), ST_Y(l.coordinate), l.address_local_language, l.address_english, n.title, c.title, 
                            r.title, s.state_name, co.name, l.postal_code, l.plus_code 
                            FROM location_view l 
                            JOIN neighborhood_ml_table n ON n.neighborhood_id = l.neighborhood_id
                            JOIN county_ml_table c ON c.county_id = l.county_id
                            JOIN region_ml_table r ON r.region_id = l.region_id
                            JOIN state_ml_table s ON s.state_id = l.state_id 
                            JOIN country_table co ON co.country_id = l.country_id 
                            WHERE l.location_id = %s LIMIT 1'''
        self.cursor.execute(select_location_sql, (location_id,))
        result = None
        coordinate = None
        result = self.cursor.fetchone()
        if result is None:
            logger.end(object={"result": result})
            return None

        (latitude, longitude, address_local_language, address_english, neighborhood,
         county, region, state, country, postal_code, plus_code) = result
        coordinate = {"latitude": latitude, "longitude": longitude}

        logger.end(object={"address_local_language": address_local_language, "address_english": address_english,
                           "neighborhood": neighborhood, "county": county, "region": region, "state": state,
                           "country": country, "postal_code": postal_code, "plus_code": plus_code})
        return (coordinate, address_local_language, address_english, neighborhood, county, region,
                state, country, postal_code, plus_code)

    def delete(self, location_id: int):
        logger.start(object={'location_id': location_id})

        update_location_sql = "UPDATE location_table SET end_timestamp = NOW() WHERE location_id = %s"
        self.cursor.execute(update_location_sql, (location_id,))

        self.connector.commit()
        logger.end()

    def _check_details_and_insert_if_not_exist(
            self, coordinate: Dict[str, float],
            location_details: tuple([str, str, str, str, str]),
            lang_code: LangCode = 'en', is_approved: bool = False, new_country_data: Dict[str, any] = None) -> tuple(
            [int, int, int, int, int]):
        logger.start(object={'coordinate': coordinate, 'location_details': location_details,
                     'lang_code': lang_code, 'new_country_data': new_country_data})
        (neighborhood_name, county_name, region_name, state_name, country_name) = location_details
        ids = self.get_location_ids(
            neighborhood_name, county_name, region_name, state_name, country_name)
        if ids is None:
            return None
        (neighborhood_ids, county_ids, region_ids, state_ids, country_id) = ids
        
        if neighborhood_ids == []:
            neighborhood_object = Neighborhood()
            neighborhood_id = neighborhood_object.insert(
                coordinate, neighborhood_name, lang_code, is_approved)
        else:
            neighborhood_id = neighborhood_ids[0]
        if county_ids == []:
            county_object = County()
            county_id = county_object.insert(coordinate, county_name, lang_code, is_approved)
        else:
            county_id = county_ids[0]
        if region_ids == []:
            region_object = Region()
            region_id = region_object.insert(coordinate, region_name, lang_code, is_approved)
        else:
            region_id = region_ids[0]
        if state_ids == []:
            state_object = State()
            state_id = state_object.insert(coordinate, state_name, lang_code)
        else:
            state_id = state_ids[0]
        #I commented the follwing lines because I don't want to insert a new country
        #if country_id == None:
            #country_object = Country()
            #country_id = country_object.insert(coordinate, country_name, lang_code, is_approved, new_country_data)
        logger.end(object={'neighborhood_id': neighborhood_id, 'county_id': county_id,
                   'region_id': region_id, 'state_id': state_id, 'country_id': country_id})
        return (neighborhood_id, county_id, region_id, state_id, country_id)

